#!/bin/bash
#

# buildGccxml.sh <product_directory> [tar]
# use the native compiler

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      (( darwin = 1 ))
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}
    sourcedir=${product_dir}/${package}/${pkgver}/source
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=gccxml
origpkgver=v0_9_20150423
pkgver=${origpkgver}
ssibuildshims_version=v0_09_00
cmake_version=v3_3_2
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgdate=`echo ${origpkgver} | cut -f3 -d"_"`

get_this_dir

get_ssibuildshims

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/bin ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir}:${PRODUCTS} ${package} ${pkgver} NO_QUAL
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# We need cmake
setup -B cmake ${cmake_version} -z ${product_dir} || die "ERROR: setup -B cmake ${cmake_version} failed"

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}
tar xf ${tardir}/gccxml-${pkgdate}.tar

if (( ${darwin:-0} )); then
  for i in 5 6 7 8 9; do
    destdir=gccxml/GCC_XML/Support/GCC/4.${i}/sys
    mkdir "${destdir}" >/dev/null 2>&1 && \
     cp "${patchdir}/cdefs.h" "${destdir}/"
  done
fi

cd ${blddir}

# be explicit about using /usr/bin/gcc on Linux
if [ ${OS1} = "Linux" ]; then
  cmake -DCMAKE_INSTALL_PREFIX=${pkgdir} -DCMAKE_C_COMPILER=/usr/bin/gcc -DCMAKE_CXX_COMPILER=/usr/bin/g++ ${sourcedir}/gccxml || die "ERROR: cmake failed."
else
  cmake -DCMAKE_INSTALL_PREFIX=${pkgdir} ${sourcedir}/gccxml || die "ERROR: cmake failed."
fi
make -j ${ncore} || die "ERROR: make failed."
make test
# Tests have been failing on OS X: ignore for now.
(( $? != 0 )) && (( ${darwin:-0} == 0 )) && die "ERROR: make test failed."
make install || die "ERROR: make test failed."

set +x

if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL 

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver}  -z ${product_dir}:${PRODUCTS}
which gccxml

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/bin ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
fi

exit 0
